module Feature.Sharing.SQLite where

import Core.Types
import Core.SQLite
import Feature.Sharing.Types

import Control.Monad.Reader           (ask, lift, liftIO)
import Crypto.Simple.CBC              (encrypt, decrypt)
import Data.Maybe                     (listToMaybe)
import Data.Time.Clock.POSIX          (getPOSIXTime)
import Database.SQLite.Simple
import Database.SQLite.Simple.FromRow

import qualified Data.ByteString.Base64 as B64
import qualified Data.ByteString.Char8  as B
import qualified Data.Text              as T
import qualified Data.Text.Encoding     as ET
import qualified Data.Text.Lazy         as LT

findByLink :: String -> PurrAction (Maybe SecretEntry)
findByLink link = do
  db   <- dbPath
  conn <- liftIO $ open db
  res  <- liftIO $ query conn "SELECT * from pws WHERE link = ?" (Only link)
  liftIO $ close conn
  return $ listToMaybe res

insertNewSecret :: T.Text -> T.Text -> PurrAction ()
insertNewSecret sec link = do
  db     <- dbPath
  key    <- encKey
  encSec <- liftIO $ encrypt (B.pack key) (ET.encodeUtf8 sec)
  conn   <- liftIO $ open db
  time   <- liftIO $ epochTime
  liftIO $ execute conn 
    "INSERT INTO pws (link, secret, date) VALUES (?, ?, ?)"
      (SecretEntry link (encodeSecret encSec) time)
  liftIO $ close conn

epochTime :: IO Integer
epochTime = fmap round getPOSIXTime

encodeSecret :: B.ByteString -> T.Text
encodeSecret b = ET.decodeUtf8 $ B64.encode b
